﻿#include <winsock2.h>
#pragma comment(lib, "Ws2_32.lib")
#include <stdio.h>
#include <locale.h>

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Function: LANPowerSample
            LAN電源への接続と調光のプログラムサンプル

  Summary:  LANPower Summary.
            LAN電源の通信プロトコル：
              ・TCPプロトコルを利用
              ・LAN電源はサーバーとして動いており、ユーザー側のプログラムの要求を受付けて処理する
              ・ユーザー側のプログラムはクライアントとして動く

            LAN電源の制御プロトコル（以下「LAN電源接続コマンド」と呼ぶ）：
            ****LAN電源接続コマンド****
            * 計4バイト
            * 1バイト目：ヘッダー、16進数固定値「7E」
            * 2バイト目：ヘッダー、16進数固定値「5A」
            * 3バイト目：LAN電源のタイプ。アナログ式-->16進数「04」、パラレール式-->16進数「05」
            * 4バイト目：LAN電源の調光値、0～255
            **********************************

  Args:     Argument1: IP Address
              引数1：LAN電源のIPアドレス
            Argument2: Port
              引数2：LAN電源のポート番号
            Argument3: Type
              引数3：LAN電源のタイプ、アナログ式またはパラレール式
                     アナログ式の場合、引数に16進数「04」をセット
                     パラレール式の場合、引数に16進数「05」をセット
            Argument4: Value
              引数4：LAN電源の調光値、0～255の値をセット

  Returns:  If successed return 0.
              成功の場合、0を返す。
            If failed return 1.
              失敗の場合、1を返す。

  Created:  2022/10/1

  Copyright © 株式会社オプター Opter Inc.
-----------------------------------------------------------------*/
int main(int argc, char** argv) {
    WSADATA wsaData;
    struct sockaddr_in serverAddr;
    SOCKET sock;
    char* serverIP;
    unsigned short serverPort;
    int iResult;
    char buf[4] = { '\0' };
    // LAN電源接続コマンドの初期化
    // 1バイト目：ヘッダー、16進数固定値「7E」をセット
    // 2バイト目：ヘッダー、16進数固定値「5A」をセット
    char cmd_table[4] = { 0x7e,0x5a,0,0 };


    // コマンドプロンプトに日本語を出力するため、システム規定のロケールに設定
    setlocale(LC_ALL, "");

    // 引数の数をチェック
    if (argc < 5) {
        wprintf(L"\nPlease check the number of the arguments. 引数の数を確認してください。\n");
        return 1;
    }

    // 引数1をIPアドレスにセット
    serverIP = argv[1];
    // 引数2をポート番号にセット
    serverPort = atoi(argv[2]);

    // 引数3をLAN電源接続コマンドの3バイト目にセット
    if (atoi(argv[3]) == 1) {
        cmd_table[2] = 0x04;
    }
    else if (atoi(argv[3]) == 2) {
        cmd_table[2] = 0x05;
    }
    else {
        wprintf(L"\nPlease input 1 or 2 to the third argument. LAN電源のタイプを「1」または「2」で入力してください。\n");
        return 1;
    }

    // 引数4の数値範囲（0～255）をチェック
    if ((atoi(argv[4]) < 0) | (atoi(argv[4]) > 255)) {
        wprintf(L"\nPlease input a value of 0 to 255 to the forth argument. 調光値に0～255の数字を入力してください。\n");
        return 1;
    }

    // 引数4をLAN電源接続コマンドの4バイト目にセット
    cmd_table[3] = atoi(argv[4]);

    // 初期化Initialize Winsock2
    iResult = WSAStartup(MAKEWORD(2, 2), &wsaData);
    if (iResult != NO_ERROR) {
        wprintf(L"\nWSAStartup failed with error: %d\n", iResult);
        return 1;
    }

    // ソケット作成Create a socket for sending data
    sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if (sock == INVALID_SOCKET) {
        wprintf(L"\nSocket failed with error: %ld\n", WSAGetLastError());
        WSACleanup();
        return 1;
    }

    // 接続先指定用構造体の準備
    memset(&serverAddr, 0, sizeof(SOCKADDR_IN));
    serverAddr.sin_family = AF_INET;
    serverAddr.sin_port = htons(serverPort);
    serverAddr.sin_addr.S_un.S_addr = inet_addr(serverIP);

    // LAN電源に接続
    if (connect(sock, (struct sockaddr*)&serverAddr, sizeof(serverAddr)) == SOCKET_ERROR) {
        wprintf(L"\nConnected failed with error:%ld. LAN電源への接続が失敗しました。\n", WSAGetLastError());
    }
    else {
        // LAN電源接続コマンドの送信
        iResult = send(sock, cmd_table, 4, 0);
        wprintf(L"Send data to the LAN Power: 0x%x 0x%x 0x%02x 0x%02x(%03d)\n", cmd_table[0], cmd_table[1], cmd_table[2], (unsigned char)cmd_table[3], (unsigned char)cmd_table[3]);

        if (iResult == 4)
        {
            int icmp;
            // LAN電源からの戻り値を受信
            int n = ::recv(sock, buf, sizeof(buf), 0);

            // LAN電源接続コマンドと戻り値を比較し、一致すれば成功となる。一致しなかったら失敗となる。
            icmp = memcmp(cmd_table, buf, 4);
            wprintf(L"\nReceive data from the LAN Power: 0x%x 0x%x 0x%02x 0x%02x(%03d)", buf[0], buf[1], buf[2], (unsigned char)buf[3], (unsigned char)buf[3]);
            if (icmp == 0 && n == 4) {
                // LAN電源への出力が成功
                wprintf(L"\nOutput succeeded. 調光値%dで出力が成功しました。\n", (unsigned char)cmd_table[3]);
            }
            else {
                // LAN電源への出力が失敗
                wprintf(L"\nOutput failed. 調光値%dで出力が失敗しました。\n", (unsigned char)cmd_table[3]);
            }
        }
        else
        {
            wprintf(L"\nOutput failed.\n");
        }
    }

    // TCPセッションの終了
    closesocket(sock);

    // winsock2の終了処理
    WSACleanup();

    return 0;
}

